/*
    <ClientSample.c>>
ңУǺץ
     ꥨ ͥåȥңУǥ <ͥåȥط饤֥ꥵץ륽>
	 $Id: LogServerSample.c,v 1.11 2004/01/24 14:42:27 yamakawa Exp $
     ------------------------------------------------------------
     Copyright (C) 2002ǯ naoki kuwamori<naoki@kuwamori.com>

     ܥץϥե꡼եȥǤʤϡFree Software Foundation
     ɽGNU ̸ͭѵΡ֥С󣲡װϤʹߤγƥС
     椫餤줫򤷡ΥС˽äܥץ
     ۤޤѹ뤳ȤǤޤ

    ʳΥ饤󥹤˾ξϡ᡼뤯.
    <naoki@kuwamori.com>


    Laurier Net work RPG server<game network library>
	 $Id: LogServerSample.c,v 1.11 2004/01/24 14:42:27 yamakawa Exp $
    Copyright (C) 2002 Naoki Kuwamori <naoki@kuwamori.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or any
    later version.

    If you wish another license . 
    mail me <naoki@kuwamori.com> .

*/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif
#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>

#include "libnet.h"
/*ϤäȤ
 * #include "laurel/libnet.h"*/

 int dummyfunc(char *command , char * readbuf , int read_len , int resfd);
 int helthcheck(char *command , char * readbuf , int read_len , int resfd);
 int chdlevel(char *command , char * readbuf , int read_len , int resfd);
 int endreq(char *command , char * readbuf , int read_len , int resfd);
 int logrequest(char *command , char * readbuf , int read_len , int resfd);
 
 #define DYMMYMES            "DYMMYMESGE"            /*ߡΥåݻ*/
 #define HELTHCHK            "HELTHCHECK"            /*إ륹å*/
 #define HELTHCHKOK          "HELTHCHKOK"            /*إ륹å*/
 #define DBGLEVEL            "DEBUGLEVEL"            /*ǥХå٥륻å׵*/
 #define ENDREQ              "ENDREQUEST"            /*λ׵*/
 #define LAURELLOGSOCNAME    "/tmp/laurellog"
 #define DEFAULTGAMENAME "laureltest"
 #define CONFIGFILENAME "config_file.csv"
 #define LAURELLOGFILENAME  "laurelLogserv.log"
 #define LIBNET_DBG_LOGFUNC        150
static int dlevel;

/*TODO ץ
 * ʴǥեեƤ
 * ¤Τ*/
typedef struct{
	char hostname[1024];
	char portnom[1024];
	char UnixSocName[1024];
	char MMapFilePath[1024];
}config_file;

/*TODO ץ
 * եեΥƥץ졼Ȥ롣*/
static CCSVTemplate tmpl_templateConfig_file[] = { 
    CSVTRECORD( offsetof( config_file, hostname),CSV_string,1023), 
    CSVTRECORD( offsetof( config_file, portnom),CSV_string,1023), 
    CSVTRECORD( offsetof( config_file, UnixSocName),CSV_string,1023),
    CSVTRECORD( offsetof( config_file, MMapFilePath),CSV_string,1023),
    EndOfCSVTemplate
};


/*Хåؿ򥻥åȤ*/
typedef struct {
	char event[11];             /* number of event */
	int (*func)();  /* function address */
}EVTBL;

/*TODO ץ
 * ٥ȥޥåפ򤳤ˤ
 * ϥ٥ȤФ륳Хåؿ򤳤Ͽ롣*/
EVTBL eventkye[20] = {
         /* ٥             ؿ*/
	{DYMMYMES,               dummyfunc},      /*ߡؿ*/
	{RPGLOGREQ,              logrequest},      /*ߡؿ*/
	{HELTHCHK,               helthcheck},     /*إ륹å׵*/
	{HELTHCHKOK,             helthcheck},     /*إ륹å*/
	{DBGLEVEL,               chdlevel},       /*ǥХå٥ѹ׵*/
	{ENDREQ,                 endreq},         /*λ׵*/
	{"",                     NULL}
};

char * gamename(char * lgamename){
    static char ggamename[MAXPATHLEN];
    static int flag=0;
    if (flag==0){
        bzero(ggamename,MAXPATHLEN);
        strncpy(ggamename,DEFAULTGAMENAME,MAXPATHLEN);
        flag=1;
    }
    if (lgamename!=NULL) strncpy(ggamename,lgamename,MAXPATHLEN);
    return ggamename;
}

char * logsockname(char * socname){
    static char gsockname[MAXPATHLEN];
    static int flag=0;
    if (flag==0){
        bzero(gsockname,MAXPATHLEN);
        strncpy(gsockname,LAURELLOGSOCNAME,MAXPATHLEN);
        flag=1;
    }
    if (socname!=NULL) strncpy(gsockname,socname,MAXPATHLEN);
    return gsockname;
}


/*ɤ߽ФƥåȤ*/
/*TODO ץ
 * ư˴ؤϤ
 * man getopt_long 򻲾Ȥ뤳*/
int set_opt(int argc, char *argv[]){
    int c;
    while (1)
    {
        int option_index = 0;
        static struct option long_options[] =
        {
           {"version", 0, 0, 'v'},
           {"help", 0, 0, 'h'},
           {"logfilename", 1, 0, 'f'},
           {"logsocname", 1, 0, 's'},
           {"debug", 1, 0, 'd'},
           {"gamename", 1, 0, 'g'},
           {0, 0, 0, 0}
        };
        c = getopt_long (argc, argv, "vhf:s:d:g:D:",
                    long_options, &option_index);
        if (c == -1) break;
        switch (c)
        {
           case 'v':
                printf ("LaurelRPG server $Id: LogServerSample.c,v 1.11 2004/01/24 14:42:27 yamakawa Exp $\n");
                exit (1);
             break;
           case 'h':
                printf ("LaurelRPG server\n");
                exit (1);
             break;
           case 'f':
                LP_setlogfile(optarg);
             break;
           case 's':
                logsockname(optarg);
             break;
           case 'D':
                dlevel=atoi(optarg);
             break;
           case 'd':
                LP_dbglvl(atoi(optarg));
             break;
           case 'g':
                gamename(optarg);
             break;

           case '?':
             break;
           default:
             printf ("?? getopt returned character code 0%o ??\n", c);
        }
    }

    if (optind < argc)
    {
     printf ("nonoption ARGVelements: ");
     while (optind < argc)
     printf ("%s ", argv[optind++]);
     printf ("\n");
    }
	return optind;
}


int main (int argc, char *argv[])
{
    char readbuf[1025];
    int read_len;
    char command[20];
    int resfd;
    int cnt;
    int UNIXserverFD;
    int ret;
	config_file testval;
	FILE * fp;

    /*ǥեͤ򥻥åȤ*/
    LP_dbglvl(101);
    LP_setlogfile(LAURELLOGFILENAME);

    /*ᤷѿ˼Ǽ*/
    set_opt(argc,argv);

    /*ե졼եɤ߹*/

    /*եΥץ*/
    if ((ret=  LP_gamelog(LIBNET_DBG_EMERG ,"start loging\n" )) < 0){
        printf("logopnen error abort\n");
        exit (0);
    }

/*TODO ץ
 * CSVǤ줿ե졼եɤ߹߽*/
	fp = fopen (CONFIGFILENAME,"r");
	if (fp==NULL){
		printf("file open error\n");
		exit(0);
	}
	while ( fgets( readbuf, sizeof( readbuf ), fp ), !feof( fp ) ) {
		if (readbuf[0]=='#')continue; /*ȤλϹԤκǽʸˡ*/
		memset (&testval , 0 ,sizeof(config_file));
		if (FALSE==GetCSV( &testval, readbuf, tmpl_templateConfig_file )){
			printf("error of read cvs file\n");
		}   
		printf("print nom[%s] string[%s] Mmap[%s]\n", testval.portnom , testval.UnixSocName,testval.MMapFilePath);
		break;
	}
/*TODO ץ
 *  * ˥ååȡ̾դѥסˤΥХץ*/
    /*ͥåȥСΥץ(˥ååȤΥץ)*/
    if ((ret = LP_serveropen( AF_UNIX ,LAURELLOGSOCNAME)) < 0){
        char errbuf[1024];
        sprintf (errbuf , "LP_serveropen error [%s] [%s] [%d]\n",testval.UnixSocName, strerror(errno) , ret);
        LP_gamelog (LIBNET_DBG_EMERG,errbuf);
        LP_endproc(LP_NG);
    }
    UNIXserverFD=ret;

/*TODO ץ*/
    /*λΤǥǡˤʤ*/
    ret = fork();
    if (ret > 0){
        if (ret==1){
            LP_gamelog (LIBNET_DBG_EMERG,"cant fork child prosses\n");
        }
        exit (1);
    }

    /*ᥤ롼*/
    while (1){
        memset (readbuf , 0 , 1024);
        ret = LP_Poll (command ,  readbuf , 1024, &read_len , 10000 , &resfd );
        readbuf[1023]=0;
        for (cnt=0;cnt<20;cnt++){
            if (eventkye[cnt].func==NULL)break;
                /*ޥʸӤפ*/
            if (strncmp(command ,eventkye[cnt].event,COMMANDLEN)==0){
                /*ХåؿƤӽФ*/
                ret = eventkye[cnt].func(command ,readbuf,read_len , resfd);
            }
        }
    }
    return LP_OK;
}

/*ߡؿ*/
int logrequest(char *command , char * readbuf , int read_len , int resfd){
    int dbglev;

	dbglev=ntohl(*((int *)readbuf));
	if (dbglev < dlevel){
		return LP_gamelog(1, &(readbuf[sizeof(int)]));
	}
	return LP_OK;
}
/*ߡؿ*/
int dummyfunc(char *command , char * readbuf , int read_len , int resfd){
    LP_gamelog(LIBNET_DBG_LOGFUNC , "dummyfunc\n");

    return LP_OK;
}
/*إ륹åؿ׵ξ*/
int helthcheck(char *command , char * readbuf , int read_len , int resfd){
    LP_gamelog(LIBNET_DBG_LOGFUNC , "helthcheck\n");

    /*ֿ*/
    if (memcmp(HELTHCHK,command,COMMANDLEN)==0){
        LP_send(resfd,HELTHCHKOK,"",0);
    }
    return LP_OK;
}
/*ǥХå٥ѹ׵*/
int chdlevel(char *command , char * readbuf , int read_len , int resfd){
    int rev;
    LP_gamelog(LIBNET_DBG_LOGFUNC , "chdlevel\n");
    /*ӥåǥб*/
    rev = ntohl (*((int *)readbuf));
    LP_dbglvl(rev);
    return LP_OK;
}
/*λ׵*/
int endreq(char *command , char * readbuf , int read_len , int resfd){
    LP_gamelog(LIBNET_DBG_LOGFUNC , "endreq\n");
    LP_fd_allclose();
    LP_gamelog(-1,"end log file\n");
	exit(0);
}

